public class ArrayBag implements Bag {

    private Book[] contents;
    private int size;

    /**
     * Creates a new ArrayBag object.
     * The initial capacity of the bag is 10.
     */
    public ArrayBag() {
        contents = new Book[10];
        size = 0;
    }

    /**
     * Adds a new book to the bag. If the contents array is full,
     * then we'll create a new array that is twice as big and copy
     * the contents of the old array into the new array.
     */
    @Override
    public void add(Book book) {
        if (size >= contents.length) {
            grow();
        }
        contents[size] = book;
        size++;
    }

    private void grow() {
        Book[] newContents = new Book[contents.length * 2];
        for (int i = 0; i < contents.length; i++) {
            newContents[i] = contents[i];
        }
        contents = newContents;
    }

    /**
     * Removes a book from the bag. If the book is not in the bag,
     * return null. Otherwise, return the book that was removed.
     * 
     * @precondition parameter target is not null
     * @precondition bag has no empty spaces up until position size
     */
    @Override
    public Book remove(Book target) {
        for (int i = 0; i < size; i++) {
            if (contents[i].equals(target)) {
                Book result = contents[i];
                contents[i] = contents[size - 1];
                size--;
                return result;
            }
        }
        return null;
    }

    /**
     * Removes a book from the bag at random. Returns null if there are no books in
     * the bag. Otherwise, return the book that was removed.
     * 
     * @precondition bag has no empty spaces up until position size
     */
    @Override
    public Book removeRandom() {

        if (size == 0) {
            return null;
        }
        int index = (int) (Math.random() * size);
        Book result = contents[index];
        contents[index] = contents[size - 1];
        size--;
        return result;
    }

    /**
     * Determines if the bag contains the specified book.
     * 
     * @precondition parameter target is not null
     */
    @Override
    public boolean contains(Book target) {
        for (int i = 0; i < size; i++) {
            if (contents[i].equals(target)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Determines if the bag contains no elements. The bag contains
     * no elements when its size is 0.
     */
    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    /**
     * Determines the number of elements currently in the bag.
     */
    @Override
    public int size() {
        return size;
    }

}
