import org.junit.Test;

import static org.junit.Assert.*;

public class ArrayBagTest {


    @Test
    public void testSizeWhenEmpty() {
        Bag b = new ArrayBag();
        int expected = 0;
        int actual = b.size();
        assertEquals(expected, actual);
    }

    @Test
    public void testIsEmptyWhenEmpty() {
        Bag b = new ArrayBag();
        assertTrue(b.isEmpty());
    }

    @Test
    public void testSizeAfterAddingABook() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Bag b = new ArrayBag();
        b.add(outline);

        int expected = 1;
        int actual = b.size();
        assertEquals(expected, actual);
    }

    @Test
    public void testIsEmptyAfterAddingABook() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Bag b = new ArrayBag();
        b.add(outline);
        assertFalse(b.isEmpty());
    }

    @Test
    public void testContainsBookAfterAdding() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Bag b = new ArrayBag();
        b.add(outline);
        assertTrue(b.contains(outline));
    }

    @Test
    public void testContainsBookEmptyBag() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Bag b = new ArrayBag();
        assertFalse(b.contains(outline));
    }

    @Test
    public void testSizeAfterAddingManyBooks() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(gilead);
        b.add(actualAir);

        int expected = 4;
        int actual = b.size();
        assertEquals(expected, actual);
    }

    @Test
    public void testContainsTrueAfterAddingManyBooks() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(gilead);
        b.add(actualAir);

        assertTrue(b.contains(gilead));
    }

    @Test
    public void testContainsFalseAfterAddingManyBooks() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(actualAir);

        assertFalse(b.contains(gilead));
    }

    @Test
    public void testRemoveReducesSize() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(gilead);

        b.remove(outline);

        int actual = b.size();
        int expected = 2;
        assertEquals(expected, actual);
    }

    @Test
    public void testRemovePresentReturnsCorrectBook() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(gilead);

        Book removed = b.remove(outline);
        Book expected = outline;
        assertEquals(expected, removed);
    }

    @Test
    public void testRemoveNotPresentReturnsNull() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(actualAir);
        b.add(gilead);

        Book removed = b.remove(outline);
        assertNull(removed);
    }

    @Test
    public void testRemoveKeepsDuplicateBook() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(actualAir);

        b.remove(actualAir);
        assertTrue(b.contains(actualAir));
    }

    @Test
    public void removeRandomReturnsNullWhenEmpty() {
        Bag b = new ArrayBag();
        Book removed = b.removeRandom();
        assertNull(removed);
    }

    @Test
    public void testRemoveRandomReducesSize() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(gilead);
        b.removeRandom();

        int expected = 2;
        int actual = b.size();
        assertEquals(expected, actual);
    }

    @Test
    public void testRemoveRandomIsNotNull() {
        Book outline = new Book("Outline", "Rachel Cusk", "9780571233625");
        Book actualAir = new Book("Actual Air", "David Berman", "81392741298");
        Book gilead = new Book("Gilead", "Marilynne Robinson", "383982892934");

        Bag b = new ArrayBag();
        b.add(outline);
        b.add(actualAir);
        b.add(gilead);
        Book removed = b.removeRandom();
        assertNotNull(removed);
    }



}