public class NodeBag implements Bag {

    private Node head;
    private int size;

    public NodeBag() {
        this.head = null;
        this.size = 0;
    }

    @Override
    public void add(Book book) {
        if (size == 0) {
            head = new Node(book);
            size = 1;
        } else {
            Node newHead = new Node(book, head);
            head = newHead;
            size++;
        }
    }


    /**
     * @param target item to be removed
     * @return the Book we removed
     */
    @Override
    public Book remove(Book target) {

        Node previous = null;
        Node current = head;

        // if bag is empty
        if (size == 0) {
            return null;
        }

        // move current to the node containing the book
        // move previous one node behind
        while (current != null && current.getBook() != target) {
            previous = current;
            current = current.getNext();
        }

        return removeHelper(current, previous);
    }

    @Override
    public Book removeRandom() {
        Node previous = null;
        Node current = head;

        int toRemove = (int) (Math.random() * size);
        int i = 0;
        while (i < toRemove) {
            previous = current;
            current = current.getNext();
            i++;
        }

        return removeHelper(current, previous);


    }

    private Book removeHelper(Node current, Node previous) {
        // case 1: couldn't find the target in the bag
        if (current == null) {
            return null;
        }
        // case 2: found the target at the head
        // unfortunately, previous is null!
        else if (current == head) {
            Book toReturn = current.getBook();
            head = head.getNext();
            size--;
            return toReturn;
        }
        // case 3: found the target somewhere else
        else {
            previous.setNext(current.getNext());
            size--;
            return current.getBook();
        }
    }

    @Override
    public boolean contains(Book target) {
        Node current = head;
        while (current != null) {
            if (current.getBook().equals(target)) {
                return true;
            }
            current = current.getNext();
        }
        return false;
    }

    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public int size() {
        return size;
    }

    public String toString() {
        String s = "{";
        Node current = head;
        while (current != null) {
            s += current.getBook() + ", ";
            current = current.getNext();
        }

        return s + '}';
    }
}
